class AccountsController < AdminController
  before_action :set_account, only: [:show, :edit, :update, :destroy]

  def index
    @accounts = paginated tabular_filter.add_keys_replacements({email: 'accounts.email', on_hold: 'accounts.on_hold',
        admin: 'if(administrators.id is not null, 1, 0)'}).apply(Account.with_admins.order :email)
  end

  def new
    @account = Account.new
  end

  def show
  end

  def edit
  end

  def create
    @account = Account.new(account_params)
    respond_to do |format|
      if @account.save
        format.html { redirect_to @account, notice: 'Account was successfully created.' }
        format.json { render :show, status: :created, location: @account }
      else
        format.html { render :new }
        format.json { render json: @account.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    respond_to do |format|
      if @account.update(account_params)
        format.html { redirect_to @account, notice: 'Account was successfully updated.' }
        format.json { head :no_content }
      else
        format.html { render action: 'edit' }
        format.json { render json: @account.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    respond_to do |format|
      if @account.administrator.nil? || @account.on_hold? || Administrator.active.count > 1
        @account.destroy
        format.html { redirect_to accounts_url, notice: 'Account was successfully destroyed.' }
        format.json { head :no_content }
      else
        format.html { redirect_to accounts_url, alert: 'You cannot destroy last active admin.' }
        format.json { head :no_content }
      end
    end
  end

  private
  # Use callbacks to share common setup or constraints between actions.
  def set_account
    @account = Account.find(params[:id])
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def account_params
    if params[:account][:password].blank?
      params[:account][:password] = nil
      params[:account][:password_confirmation] = nil
    end
    params.require(:account).permit(:email, :investment, :percentage_attendant, :on_hold, :subscribed, :admin, :password, :password_confirmation)
  end
end
