class ApplicationController < ActionController::Base
  protect_from_forgery with: :exception
  before_action :check_administrator, unless: :devise_controller?
  before_filter :set_locale

  helper_method :current_url_for_locale
  def current_url_for_locale(locale)
    request.get? ? url_for(params.merge(locale: locale)) : url_for(locale: locale)
  end

  protected
  def check_administrator
    if current_account.nil?
      authenticate_account!
    elsif current_account.administrator.nil?
      redirect_to root_url, alert: 'You have not enough rights to do it!'
    end
  end

  def extract_best_locale
    if request.env['HTTP_ACCEPT_LANGUAGE']
      return request.env['HTTP_ACCEPT_LANGUAGE'].scan(/^[a-z]{2}/).first || I18n.default_locale
    end
    I18n.default_locale
  end

  def set_locale
    ActionMailer::Base.default_url_options = {:host => request.host_with_port}
    params[:locale] = extract_best_locale if params[:locale].nil?
    if I18n.available_locales.include?(params[:locale].to_sym)
      current_account.update(locale: params[:locale]) if current_account && current_account.locale != params[:locale]
      I18n.locale = params[:locale]
    else
      flash.now[:notice] = "#{params[:locale]} translation not available"
      logger.error flash.now[:notice]
    end
  end

  def self.default_url_options(options={})
    options.merge({locale: I18n.available_locales.length > 1 ? I18n.locale: nil})
  end

end
