class BlocksController < AdminController
  before_action :set_block, only: [:edit, :update]
  layout false

  def edit
  end

  def update
    replace_line_breaks
    update_block_by_version
    respond_to do |format|
      format.js
    end
  end

  def apply
    BlockVersion.by_admin(current_account).last.activate
    respond_to do |format|
      format.js { render nothing: true, status: 200 }
      format.html { redirect_to :back }
    end
  end

  def cancel
    last_block_version = BlockVersion.by_admin(current_account).last
    if last_block_version.present? and !last_block_version.active
      last_block_version.blocks.destroy_all
    end
    respond_to do |format|
      format.js { render nothing: true, status: 200 }
      format.html { redirect_to :back }
    end
  end

  private
  def set_block
    @block = Block.last_for_admin(params[:id], current_account)
  end

  def block_params
    params.require(:block).permit(:key, localed_attr(:text))
  end

  def replace_line_breaks
    locales.each do |locale|
      split_text = params[:block]["text_#{locale}"].split(/\r\n|\n\r|\r|\n/)
      params[:block]["text_#{locale}"] = split_text.map {|p| p.blank? ? '<br>' : "<p>#{p}</p>"}.join
    end
  end

  def update_block_by_version
    full_params = block_params
    full_params[:key] = params[:id]
    if @block.block_version.active?
      block_version = BlockVersion.find_or_create_by(account: current_account, active: false)
      @block = Block.new(key: params[:id], block_version: block_version)
      @prev_block = Block.active.by_key(params[:id]).last
      if @prev_block
        @prev_block.text_phrase.translations.each do |translation|
          full_params["text_#{translation.locale}"] = translation.text if translation.locale != I18n.locale.to_s
        end
      end
    end
    @block.update(full_params)
  end

end
