class TabularFilter
  include ActionView::Helpers

  def initialize(params)
    filters_params = params.select{|key| key.to_s.match(/tf.?_.+/)}
    @filters = Hash[filters_params.map{|k,v| [k.to_s.match(/tf.?_(.+)/)[1],{value: v, mode: k.to_s.match(/tf(.?)_.+/)[1]}]}]
    @keys_replacements = {}
    @joins = {}
  end

  def translations(*translated_columns)
    relations Hash[translated_columns.map { |c| [c, '.:t'] }]
  end

  def relations(args)
    # EXAMPLE: {manager: 'accounts.name', client: 'accounts.name', ware: 'wares.name:t', name: '.:t'}
    args.keys.each do |key|
      value = args[key].split('.')
      table_name = value[0]
      join_table_name = key.to_s.pluralize
      column_params = value[1].split(':')
      column_name = column_params[0]
      translatable = false
      if column_params.length > 1 && column_params[1]=='t'
        translatable = true
        join_table_name += "_#{column_name}_translations"
      end
      @joins[key] = {table_name: table_name, join_table_name: join_table_name, column: key, join_column: column_name,
          translatable: translatable}
      add_keys_replacements(key => "#{join_table_name}.#{translatable ? 'text' : column_name}")
    end
    self
  end

  def add_keys_replacements(keys)
    @keys_replacements.merge! keys
    self
  end

  def apply(scope, args = {})
    decode_apply_args(args)
    @table_name = scope.table.name
    active? ? add_joins(scope).where(condition).uniq : scope
  end

  def decode_apply_args(args)
    @skip = []
    if args.has_key? :skip
      if args[:skip].is_a? Array
        args[:skip].each { |value| @skip << value }
      else
        @skip << args[:skip].to_sym
      end
    end
  end

  def add_joins(scope)
    @joins.each do |key, join_data|
      next if @skip.include? key.to_sym
      jt_column = join_data[:join_column]
      jt_name = join_data[:join_table_name]
      if jt_column.empty?
        jt_column = join_data[:column]
        jt = @table_name
        column = 'id'
      else
        jt = join_data[:table_name]
        column = join_data[:column].to_s + '_id'
      end
      if join_data[:translatable]
        scope = scope.
            joins "LEFT JOIN phrase_containers `#{jt}_containers` ON `#{jt}_containers`.`translatable_id` = `#{@table_name}`.`#{column}` AND `#{jt}_containers`.`translatable_type`='#{jt.classify}' AND `#{jt}_containers`.`attribute_name`='#{jt_column}'",
                "LEFT JOIN `phrases` `#{jt}_phrases` ON `#{jt}_phrases`.`id` = `#{jt}_containers`.`phrase_id`",
                "LEFT JOIN `translations` `#{jt_name}` ON `#{jt_name}`.`phrase_id` = `#{jt}_phrases`.`id` AND `#{jt_name}`.`locale` = '#{I18n.locale}'"
      else
        scope = scope.
            joins "LEFT JOIN `#{jt}` `#{jt_name}` ON `#{jt_name}`.`id` = `#{@table_name}`.`#{column}`"
      end
    end
    scope
  end

  def condition
    condition = ''
    @filters.keys.each do |key|
      return 'false' if @skip.include? key.to_sym
      condition << ' and ' unless condition.blank?
      condition << key_condition(key)
    end
    condition
  end

  def key_condition(key)
    @key = key
    condition = ''
    sub_keys = key.split(':').map(&:to_sym).map { |k| @keys_replacements.include?(k) ? @keys_replacements[k] : k }
    (0..sub_keys.length - 1).each do
      condition << ' or ' unless condition.blank?
      condition << current_keys_condition(sub_keys)
      sub_keys << sub_keys.shift
    end
    "(#{condition})"
  end

  def current_keys_condition(keys)
    condition = ''
    values = @filters[@key][:value].split(' ')
    (0..values.length - 1).each do
      condition << ' or ' unless condition.blank?
      condition << current_values_condition(keys, values)
      values << values.shift
    end
    condition
  end

  def current_values_condition(keys, values)
    condition = ''
    (0..[values.length, keys.length].min - 1).each do |i|
      condition << ' and ' unless condition.blank?
      condition << value_condition(keys[i], values[i])
    end
    "(#{condition})"
  end

  #noinspection RubyAssignmentExpressionInConditionalInspection
  def value_condition(key, value)
    col_name = key.to_s
    col_name = "#{@table_name}.#{col_name}" if col_name.gsub(/\./).count == 0
    case @filters[@key][:mode]
      when 'd'
        date = value.to_datetime
        "(#{col_name} BETWEEN '#{date.beginning_of_day.to_s(:db)}' AND '#{date.end_of_day.to_s(:db)}')"
      when 'b'
        value == '1' ? "(#{col_name} = 1)" : "(#{col_name} = 0 OR #{col_name} IS NULL)"
      else
        if value == '!*'
          return "(#{col_name} is null or #{col_name}='')"
        elsif value == '*'
          return "(#{col_name} is not null and #{col_name}<>'')"
        elsif match = /^(==)(\d*.\d*)$/.match(value)
          return "(#{col_name}=#{match[2]})"
        elsif match = /^(<>)(\d*.\d*)$/.match(value)
          return "(#{col_name}!=#{match[2]})"
        elsif match = /^(!=|[><=])(\d*.\d*)$/.match(value)
          return "(#{col_name}#{match[1]}#{match[2]})"
        end
        "#{col_name} like '%#{value}%'"
    end
  end

  def active?
    !@filters.empty?
  end

  def empty?
    @filters.empty?
  end

  def row_helper(*args)
    out = "<tr id='filters'>"
    put = true
    args = args[0] if args[0].is_a? Array
    args.each_with_index do |param, i|
      param = 1 if param == '-' || param == ''
      if param.is_a? Integer
        if i == args.size - 1
          if block_given?
            out << "<th class='actions' colspan='#{param}'>#{yield}</th>"
          else
            out << "<th class='actions' colspan='#{param}'>#{link_to '', '?', class: 'clear_btn clear_tabular_filters',
                title: t('helpers.links.clear', default: 'Clear')}</th>"
          end
        else
          out << "<th colspan='#{param}'></th>" if put
        end
        put = true
      elsif !!param == param
        put = param
      elsif param.is_a? Hash
        param.keys.each do |key|
          out << generate_control(param[key], key) if put
        end
      else
        param_parts = param.to_s.split(':')
        type = param_parts.length > 1 ? param_parts[1].to_sym : :text
        out << generate_control(type, param_parts[0]) if put
        put = true
      end
    end
    out << '</tr>'
    out.html_safe
  end

  private

  def generate_control(type, param)
    case type
      when :text, :string, :as_text, :as_string
        "<th>#{field_helper param}</th>"
      when :bool, :boolean, :as_bool, :as_boolean
        "<th>#{yes_no_helper param}</th>"
      when :date, :as_date
        "<th>#{date_helper param}</th>"
      else
        '<th></th>'
    end
  end

  def field_helper(attr)
    text_field_tag attr, value_for(attr), class: 'filter'
  end

  def yes_no_helper(attr)
    select_tag attr, options_for_select([[t('humanizers.bool.true'), 1], [t('humanizers.bool.false'), 0]], value_for(attr)),
        include_blank: true, class: 'filter boolean'
  end

  def date_helper(attr)
    text_field_tag attr, value_for(attr), class: 'filter date_picker'
  end

  def value_for(attr)
    @filters.has_key?(attr) ? @filters[attr][:value] : ''
  end

end
