class NewslettersController < AdminController
  before_action :set_newsletter, only: [:show, :edit, :update, :destroy, :sending, :resend]

  # GET /newsletters
  # GET /newsletters.json
  def index
    @newsletters = Newsletter.all
  end

  # GET /newsletters/1
  # GET /newsletters/1.json
  def show
  end

  # GET /newsletters/new
  def new
    @newsletter = params.has_key?(:newsletter) ? Newsletter.new(newsletter_params) : Newsletter.new
  end

  # GET /newsletters/1/edit
  def edit
  end

  # POST /newsletters
  # POST /newsletters.json
  def create
    @newsletter = Newsletter.new(newsletter_params)

    respond_to do |format|
      if @newsletter.save
        format.html { redirect_to @newsletter, notice: 'Newsletter was successfully created.' }
        format.json { render :show, status: :created, location: @newsletter }
      else
        format.html { render :new }
        format.json { render json: @newsletter.errors, status: :unprocessable_entity }
      end
    end
  end

  # PATCH/PUT /newsletters/1
  # PATCH/PUT /newsletters/1.json
  def update
    respond_to do |format|
      if @newsletter.update(newsletter_params)
        format.html { redirect_to @newsletter, notice: 'Newsletter was successfully updated.' }
        format.json { render :show, status: :ok, location: @newsletter }
      else
        format.html { render :edit }
        format.json { render json: @newsletter.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /newsletters/1
  # DELETE /newsletters/1.json
  def destroy
    @newsletter.destroy
    respond_to do |format|
      format.html { redirect_to newsletters_url, notice: 'Newsletter was successfully destroyed.' }
      format.json { head :no_content }
    end
  end

  def resend
    @newsletter.update(last_id: nil)
    redirect_to send_newsletter_url
  end

  def sending
    if @newsletter.last_id.nil?
      init_newsletter
    elsif @newsletter.processed_id < @newsletter.last_id
      sending_newsletter
    end
  end

  private
  def init_newsletter
    @newsletter.update(last_id: filtered_accounts.last.id, processed_id: 0)
  end

  def filtered_accounts
    Account.where(subscribed: true).for_filter.where(@newsletter.recipient_filter).order(:id)
  end

  def sending_newsletter
    @accounts = filtered_accounts.where('accounts.id > ?', @newsletter.processed_id).first(100)
    begin
      FrontMailer.newsletter(@newsletter.text_for_request(request), @accounts.map(&:email), @newsletter.subject).deliver_now
    rescue Exception => error
      Rails.logger.error("Error on send newsletter to #{error.message}")
    end
    params = {processed_id: @accounts.last.id}
    if @accounts.last.id >= @newsletter.last_id
      params[:processed_at] = DateTime.current
      params[:last_id] = @accounts.last.id
    end
    @newsletter.update(params)
  end

  # Use callbacks to share common setup or constraints between actions.
  def set_newsletter
    @newsletter = Newsletter.find(params[:id])
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def newsletter_params
    params.require(:newsletter).permit(:subject, :text, :date, :recipient_filter)
  end
end
